Package: bsub
Type: Package
Title: Submitter and Monitor of the 'LSF Cluster'
Version: 2.0.7
Date: 2026-04-08
Authors@R: person("Zuguang", "Gu", email = "guzuguang@suat-sz.edu.cn", role = c("aut", "cre"),
                  comment = c('ORCID'="0000-0002-7395-8709"))
Depends: R (>= 4.0.0)
Imports: GlobalOptions (>= 0.1.1), GetoptLong (>= 0.1.8), digest,
        utils, stats, clisymbols, crayon, grDevices, graphics,
        codetools, ssh, igraph
Suggests: knitr, DT (>= 0.13), shiny (>= 1.0.0), shinyjs, graph,
        ggplot2, rmarkdown, markdown, DiagrammeR
biocViews: Software, Infrastructure
Description: It submits R code/R scripts/shell commands to 'LSF cluster' 
  (<https://en.wikipedia.org/wiki/Platform_LSF>, the 'bsub' system) without 
  leaving R. There is also an interactive 'shiny' application for monitoring job status.
VignetteBuilder: knitr
URL: https://github.com/jokergoo/bsub
License: MIT + file LICENSE
NeedsCompilation: no
SystemRequirements: Platform LSF, bsub
Encoding: UTF-8
RoxygenNote: 7.3.1
Packaged: 2026-04-08 04:52:52 UTC; guz
Author: Zuguang Gu [aut, cre] (ORCID: <https://orcid.org/0000-0002-7395-8709>)
Maintainer: Zuguang Gu <guzuguang@suat-sz.edu.cn>
Repository: CRAN
Date/Publication: 2026-04-08 09:40:14 UTC
Built: R 4.6.0; ; 2026-04-26 11:56:45 UTC; unix
