% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bjobs.R
\name{bjobs_reset_timestamp}
\alias{bjobs_reset_timestamp}
\title{Clear job history}
\usage{
bjobs_reset_timestamp()
}
\description{
Clear job history
}
\details{
It sets a timestamp to only show jobs after it.
}
