% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bjobs.R
\name{print.bjobs}
\alias{print.bjobs}
\title{Summary of jobs}
\usage{
\method{print}{bjobs}(x, ...)
}
\arguments{
\item{x}{A \code{bjobs} object.}

\item{...}{other arguments.}
}
\description{
Summary of jobs
}
